import typer
from pytronic.commands.start import Start
from pytronic.commands.createbot import CreateBot
from pytronic.commands.run import Run

app = typer.Typer()

Start.register(app)
CreateBot.register(app)
Run.register(app)

if __name__ == '__main__':
    app()



import pyautogui


class Mouse:
    @staticmethod
    def move_to(x, y):
        """Move the mouse to (x, y)."""
        pyautogui.moveTo(x, y)
        return Mouse

    @staticmethod
    def click_position(x=None, y=None, clicks=1, button='left'):
        """Click at (x, y)."""
        pyautogui.click(x=x, y=y, clicks=clicks, button=button)
        return Mouse

    @staticmethod
    def drag_to(x, y, button='left'):
        """Drag the mouse to (x, y)."""
        pyautogui.dragTo(x, y, button=button)
        return Mouse

    @staticmethod
    def click_in_window_close_button():
        """Click the window close button."""
        close_button_720p = (1270, 10)
        close_button_1080p = (1870, 10)

        screen_width, screen_height = pyautogui.size()

        if screen_height <= 720:
            close_button_position = close_button_720p
        elif screen_height <= 1080:
            close_button_position = close_button_1080p
        else:
            close_button_position = (screen_width - 50, 10)

        pyautogui.moveTo(*close_button_position)
        pyautogui.click(*close_button_position)
        return Mouse

    @staticmethod
    def click_image(image_path, clicks=1, button='left'):
        """Click on the image found on the screen."""
        location = pyautogui.locateOnScreen(image_path)
        if location:
            x, y = pyautogui.center(location)
            pyautogui.click(x=x, y=y, clicks=clicks, button=button)
            return Mouse
        raise ValueError(f'Image not found on screen: {image_path}')

    @staticmethod
    def move_to_image(image_path):
        """Move the mouse to the image found on the screen."""
        location = pyautogui.locateOnScreen(image_path)
        if location:
            x, y = pyautogui.center(location)
            pyautogui.moveTo(x, y)
            return Mouse
        raise ValueError(f'Image not found on screen: {image_path}')

    @staticmethod
    def drag_and_drop(start, end):
        """Drag from start to end position."""
        pyautogui.moveTo(*start)
        pyautogui.dragTo(*end)
        return Mouse

    @staticmethod
    def scroll(amount):
        """Scroll the screen."""
        pyautogui.scroll(amount)
        return Mouse



import os, json, importlib, inspect
from pydantic import BaseModel
from pytronic.utils import convert_to_snake_case
from pytronic.management.file_manager import FileManager
from pytronic.bot import Bot


class BotLoader:
    def __init__(self, bot_name: str, task: str = None, file_manager=None):
        self.file_manager = file_manager or FileManager()
        self._bot_name = bot_name
        self._task = task

    def get(self) -> Bot:
        bot_name_snake_case = convert_to_snake_case(self._bot_name)
        self.file_manager.validate_bots_directory()

        bot_module_path = self.file_manager.get_bot_module_path(
            bot_name_snake_case
        )

        self.file_manager.check_bot_exists(bot_module_path, self._bot_name)
        bot_dict = self._load_bots(bot_name_snake_case)
        task_schema = self._load_task_schema(bot_name_snake_case)
        return self._get_bot_instance(
            bot_dict, bot_name_snake_case, task_schema
        )

    def _load_task_schema(self, bot_name_snake_case: str):
        task_module_name = (
            f'{self.file_manager.base_dir}.{bot_name_snake_case}.task'
        )
        task_module = importlib.import_module(task_module_name)
        for name, obj in inspect.getmembers(task_module):
            if (
                inspect.isclass(obj)
                and issubclass(obj, BaseModel)
                and obj is not BaseModel
            ):
                return obj
        raise Exception(f'Task schema not found for bot {bot_name_snake_case}')

    def _load_bots(self, bot_name_snake_case: str):
        module_name = f'{bot_name_snake_case}.{bot_name_snake_case}'
        module = importlib.import_module(module_name)
        bots = self._get_bot_classes(module)
        return {bot.__name__: bot for bot in bots}

    @staticmethod
    def _get_bot_classes(module):
        return [
            obj
            for _, obj in inspect.getmembers(module)
            if inspect.isclass(obj) and issubclass(obj, Bot) and obj is not Bot
        ]

    def _get_bot_instance(
        self, bot_dict: dict, bot_name_snake_case: str, task_schema: BaseModel
    ):
        if self._bot_name in bot_dict:
            task_data = self._load_task_data(bot_name_snake_case)
            bot_strategy = bot_dict[self._bot_name]
            return bot_strategy(task=task_schema(**task_data))
        raise Exception(f'Bot {self._bot_name} not found.')

    def _load_task_data(self, bot_name_snake_case: str) -> dict:
        if self._task:
            return self._load_task_from_string_or_file(self._task)
        return self.file_manager.load_default_task_file(bot_name_snake_case)

    @staticmethod
    def _load_task_from_string_or_file(task: str) -> dict:
        if os.path.isfile(task):
            with open(task, 'r') as f:
                return json.load(f)
        try:
            task_string = task.replace("'", '"')
            return json.loads(task_string)
        except json.JSONDecodeError:
            raise Exception(
                'The --task argument is not a valid JSON or an existing file path.'
            )



import typer


class Command:
    @classmethod
    def register(cls, app: typer.Typer):
        command_instance = cls(app)
        command_name = cls.__name__.replace('Command', '').lower()
        app.command(name=command_name)(command_instance.execute)

    def __init__(self, app: typer.Typer):
        self.app = app

    def execute(self):
        raise NotImplementedError(
            'Subclasses must implement the execute method.'
        )



import pyautogui
from contextlib import contextmanager


class Keyboard:
    @staticmethod
    def type_text(text, interval=None):
        """Type the given text."""
        pyautogui.typewrite(text, interval=interval)
        return Keyboard

    @staticmethod
    def press_key(key):
        """Press a specific key."""
        pyautogui.press(key)
        return Keyboard

    @staticmethod
    def hotkey(*keys):
        """Press a combination of keys."""
        pyautogui.hotkey(*keys)
        return Keyboard

    @staticmethod
    def press_left():
        """Press the left arrow key."""
        pyautogui.press('left')
        return Keyboard

    @staticmethod
    def press_right():
        """Press the right arrow key."""
        pyautogui.press('right')
        return Keyboard

    @staticmethod
    def press_up():
        """Press the up arrow key."""
        pyautogui.press('up')
        return Keyboard

    @staticmethod
    def press_down():
        """Press the down arrow key."""
        pyautogui.press('down')
        return Keyboard

    @staticmethod
    @contextmanager
    def hold_key(key):
        """Hold a specific key."""
        pyautogui.keyDown(key)
        try:
            yield
        finally:
            pyautogui.keyUp(key)

    @staticmethod
    @contextmanager
    def hold_ctrl():
        """Hold the Ctrl key."""
        pyautogui.keyDown('ctrl')
        try:
            yield
        finally:
            pyautogui.keyUp('ctrl')

    @staticmethod
    @contextmanager
    def hold_alt():
        """Hold the Alt key."""
        pyautogui.keyDown('alt')
        try:
            yield
        finally:
            pyautogui.keyUp('alt')

    @staticmethod
    @contextmanager
    def hold_shift():
        """Hold the Shift key."""
        pyautogui.keyDown('shift')
        try:
            yield
        finally:
            pyautogui.keyUp('shift')

    @staticmethod
    def copy():
        """Copy selected text."""
        Keyboard.hotkey('ctrl', 'c')
        return Keyboard

    @staticmethod
    def paste():
        """Paste copied text."""
        Keyboard.hotkey('ctrl', 'v')
        return Keyboard

    @staticmethod
    def select_all():
        """Select all text."""
        Keyboard.hotkey('ctrl', 'a')
        return Keyboard



import typer, os
from pytronic.commands.base.command import Command
from pytronic.management import FileManager


class Start(Command):
    def execute(self):
        """Initializes the project by creating the necessary folders and an example bot."""
        if not os.path.exists('bots'):
            os.makedirs('bots')
            typer.echo("path 'bots/' created.")

        FileManager().create_dir_bot('Example')



import time
from pytronic.components import Keyboard, Mouse, Screen


class ComputerInteraction:
    def __init__(self):
        self.keyboard = Keyboard()
        self.mouse = Mouse()
        self.screen = Screen()

    def wait_for_image_and_click(self, image_name, timeout=None):
        """Wait for an image to appear on the screen and click it."""
        start_time = time.time()
        while timeout is None or (time.time() - start_time) <= timeout:
            try:
                x, y = self.screen.locate_image(image_name)
                self.mouse.click_position(x, y)
                return
            except ValueError:
                if timeout and (time.time() - start_time) > timeout:
                    raise TimeoutError(
                        f'Image {image_name} not found within {timeout} seconds.'
                    )
                time.sleep(1)
        raise TimeoutError(
            f'Image {image_name} not found within {timeout} seconds.'
        )

    def wait_for_image_to_disappear(self, image_name, timeout=None):
        """Wait for an image to disappear from the screen."""
        start_time = time.time()
        while timeout is None or (time.time() - start_time) <= timeout:
            try:
                self.screen.locate_image(image_name)
                if timeout and (time.time() - start_time) > timeout:
                    raise TimeoutError(
                        f'Image {image_name} did not disappear within {timeout} seconds.'
                    )
                time.sleep(1)
            except ValueError:
                return
        raise TimeoutError(
            f'Image {image_name} did not disappear within {timeout} seconds.'
        )

    def type_text_when_image_appears(self, image_name, text, timeout=None):
        """Type text when a specific image appears on the screen."""
        self.wait_for_image_and_click(image_name, timeout=timeout)
        self.keyboard.type_text(text)

    def click_image_until_disappears(self, image_name, timeout=None):
        """Click on an image repeatedly until it disappears from the screen or the timeout is reached."""
        start_time = time.time()
        while timeout is None or (time.time() - start_time) <= timeout:
            try:
                x, y = self.screen.locate_image(image_name)
                self.mouse.click_position(x, y)
                time.sleep(0.5)  # Adjust sleep duration as needed
            except ValueError:
                return
        raise TimeoutError(
            f'Image {image_name} did not disappear within {timeout} seconds.'
        )

    def wait_for_text_and_type(self, text, input_text, timeout=None):
        """Wait for specific text to appear on the screen and type the given input text."""
        start_time = time.time()
        while timeout is None or (time.time() - start_time) <= timeout:
            screenshot_path = self.screen.take_screenshot_auto_name()
            detected_text = self.screen.ocr_image(screenshot_path.name)
            if text in detected_text:
                self.keyboard.type_text(input_text)
                return
            time.sleep(1)
        raise TimeoutError(f'Text {text} not found within {timeout} seconds.')

    def scroll_until_image_visible(
        self, image_name, direction='down', timeout=None
    ):
        """Scroll the screen until a specific image is visible or the timeout is reached."""
        start_time = time.time()
        while timeout is None or (time.time() - start_time) <= timeout:
            try:
                self.screen.locate_image(image_name)
                return
            except ValueError:
                if direction == 'down':
                    self.mouse.scroll(-10)  # Scroll down
                elif direction == 'up':
                    self.mouse.scroll(10)  # Scroll up
                time.sleep(0.5)  # Adjust sleep duration as needed
        raise TimeoutError(
            f'Image {image_name} not found within {timeout} seconds.'
        )

    def wait_for_image_and_double_click(self, image_name, timeout=None):
        """Wait for an image to appear on the screen and double click it."""
        start_time = time.time()
        while timeout is None or (time.time() - start_time) <= timeout:
            try:
                x, y = self.screen.locate_image(image_name)
                self.mouse.click_position(x, y, clicks=2)
                return
            except ValueError:
                if timeout and (time.time() - start_time) > timeout:
                    raise TimeoutError(
                        f'Image {image_name} not found within {timeout} seconds.'
                    )
                time.sleep(1)
        raise TimeoutError(
            f'Image {image_name} not found within {timeout} seconds.'
        )

    def capture_screenshot_when_image_appears(self, image_name, timeout=None):
        """Capture a screenshot when a specific image appears on the screen."""
        start_time = time.time()
        while timeout is None or (time.time() - start_time) <= timeout:
            try:
                self.screen.locate_image(image_name)
                screenshot_path = self.screen.take_screenshot_auto_name()
                return screenshot_path
            except ValueError:
                if timeout and (time.time() - start_time) > timeout:
                    raise TimeoutError(
                        f'Image {image_name} not found within {timeout} seconds.'
                    )
                time.sleep(1)
        raise TimeoutError(
            f'Image {image_name} not found within {timeout} seconds.'
        )



import os
import json
import sys
from jinja2 import Environment, FileSystemLoader
from pytronic.utils import convert_to_snake_case
from importlib.resources import files


class FileManager:
    def __init__(self, base_dir='bots', template_dir='pytronic/templates'):
        self.base_dir = base_dir
        self.template_dir = template_dir

    def create_dir_bot(self, name: str, base_path=None):
        self._bot_name = name
        bot_name_snake_case = convert_to_snake_case(name)
        bot_dir = self._ensure_directory_exists(bot_name_snake_case)
        self._create_bot_file(bot_dir, bot_name_snake_case, name)
        self._create_task_json(bot_dir, base_path=base_path)
        self._create_task_schema(bot_dir)
        self._create_settings()

    def get_dir_bot(self):
        try:
            return f'{self.base_dir}/{convert_to_snake_case(self._bot_name)}'
        except AttributeError:
            return None

    def _create_task_schema(self, bot_dir):
        task_filename = os.path.join(bot_dir, 'task.py')
        template_content = self._load_template('task_template.py.jinja')
        task_content = template_content.render()

        with open(task_filename, 'w') as f:
            f.write(task_content)

        return task_filename

    def _create_task_json(self, bot_dir, base_path=None):
        if base_path:
            bot_dir = os.path.join(base_path, bot_dir)
        task_filename = os.path.join(bot_dir, 'task.json')
        os.makedirs(bot_dir, exist_ok=True)
        if not os.path.exists(task_filename):
            with open(task_filename, 'w') as f:
                json.dump({}, f)

    def _ensure_directory_exists(self, sub_dir: str) -> str:
        base_path = os.path.join(self.base_dir)

        if not os.path.exists(base_path):
            os.makedirs(base_path)

        sub_path = os.path.join(base_path, sub_dir)
        os.makedirs(sub_path, exist_ok=True)
        return sub_path

    def _create_bot_file(
        self, bot_dir: str, bot_name_snake_case: str, bot_classname: str
    ) -> str:
        bot_filename = os.path.join(bot_dir, f'{bot_name_snake_case}.py')
        template_content = self._load_template('bot_template.py.jinja')
        bot_content = template_content.render(bot_classname=bot_classname)

        with open(bot_filename, 'w') as file:
            file.write(bot_content)

        return bot_filename

    def validate_bots_directory(self):
        current_directory = os.getcwd()
        bots_directory = os.path.join(current_directory, self.base_dir)

        if not os.path.exists(bots_directory):
            raise Exception(
                'The "bots" directory does not exist. Please run "pytronic init" first.'
            )

        sys.path.insert(0, bots_directory)

    def get_bot_module_path(self, bot_name_snake_case: str) -> str:
        return os.path.join(
            self.base_dir, bot_name_snake_case, f'{bot_name_snake_case}.py'
        )

    def check_bot_exists(self, bot_module_path: str, bot_name: str):
        if not os.path.exists(bot_module_path):
            raise Exception(
                f'Bot {bot_name} not found at the expected path: {bot_module_path}'
            )

    def load_default_task_file(self, bot_name_snake_case: str):
        task_file = os.path.join(
            self.base_dir, bot_name_snake_case, 'task.json'
        )

        if not os.path.exists(task_file):
            raise Exception(
                f'No task provided and the default file "{task_file}" was not found.'
            )

        with open(task_file, 'r') as f:
            return json.load(f)

    def _create_settings(self):
        settings_filename = os.path.join(self.base_dir, 'settings.py')
        template_content = self._load_template('settings.py.jinja')
        settings_content = template_content.render(
            screenshots_path='images/screenshots',
            images_path='images',
            ocr_language='eng',
            screen_width=1920,
            screen_height=1080,
            pytesseract_path='/usr/bin/tesseract',
        )

        with open(settings_filename, 'w') as file:
            file.write(settings_content)

        return settings_filename

    def _load_template(self, template_name):
        template_path = files(self.template_dir.replace('/', '.')).joinpath(
            template_name
        )
        with template_path.open('r') as file:
            content = file.read()
        return Environment().from_string(content)



from pytronic.management import BotLoader
from pytronic.bot import Bot
from pytronic.management import FileManager


class BotFactory:
    def __init__(self, bot_name: str, task: str | dict = None):
        self.bot_name = bot_name
        self.task = task if not isinstance(task, dict) else str(task)

    def create(self, dir_exists=False) -> Bot:
        if not dir_exists:
            FileManager().create_dir_bot(self.bot_name)
        return BotLoader(self.bot_name, self.task).get()



from pydantic import BaseModel


class Bot:
    def __init__(self, task: BaseModel):
        self.task = task

    def run(self):
        raise NotImplementedError(
            'O método run deve ser implementado pela subclasse'
        )



import typer
from pytronic.commands.base.command import Command
from pytronic.factory.bot_factory import BotFactory


class Run(Command):
    def execute(
        self,
        bot_name: str,
        task: str = typer.Option(None, help='Task for the bot to execute'),
    ):
        """Executes a bot with the specified task."""
        bot = BotFactory(bot_name, task).create(dir_exists=True)
        bot.run()



import typer
from pytronic.commands.base.command import Command
from pytronic.management import FileManager


class CreateBot(Command):
    def execute(self, name: str):
        """Creates a new bot with the specified name."""
        FileManager().create_dir_bot(name)
        typer.echo(f'Created in /bots a new bot: {name}')



import pyautogui
import pytesseract
import time
from PIL import Image, ImageDraw
from datetime import datetime
from bots.settings import (
    SCREENSHOTS_PATH,
    IMAGES_PATH,
    OCR_LANGUAGE,
    PYTESSERACT_PATH,
)


class Screen:
    def __init__(self):
        self.screenshots_path = SCREENSHOTS_PATH
        self.images_path = IMAGES_PATH
        self.ocr_language = OCR_LANGUAGE
        pytesseract.pytesseract.tesseract_cmd = PYTESSERACT_PATH

    def take_screenshot(self, path, region=None):
        """Takes a screenshot and saves it to the specified path.
        If region is specified, it should be a tuple: (x, y, width, height).
        """
        pyautogui.screenshot(path, region=region)

    def take_screenshot_auto_name(self, region=None):
        """Takes a screenshot and saves it with an automatically generated name based on the current date and time.
        If region is specified, it should be a tuple: (x, y, width, height).
        """
        if not self.screenshots_path.exists():
            self.screenshots_path.mkdir(parents=True)
        timestamp = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')
        path = self.screenshots_path / f'screenshot_{timestamp}.png'
        self.take_screenshot(str(path), region=region)
        return path

    def take_screenshot_with_highlight(self, path, region):
        """Takes a full-screen screenshot and highlights a specified region.
        Region should be a tuple: (x, y, width, height).
        """
        screenshot = pyautogui.screenshot()
        img = Image.fromarray(screenshot)
        draw = ImageDraw.Draw(img)
        draw.rectangle(region, outline='red', width=5)
        img.save(path)

    def locate_image(self, image_name):
        """Locate an image on the screen and return its position."""
        image_path = self.images_path / image_name
        location = pyautogui.locateOnScreen(str(image_path))
        if location:
            return pyautogui.center(location)
        raise ValueError(f'Image not found on screen: {image_path}')

    def locate_all_images(self, image_name):
        """Locate all occurrences of an image on the screen and return their positions."""
        image_path = self.images_path / image_name
        locations = pyautogui.locateAllOnScreen(str(image_path))
        centers = [pyautogui.center(loc) for loc in locations]
        return centers

    def ocr_image(self, image_name):
        """Perform OCR on a given image and return the extracted text."""
        image_path = self.images_path / image_name
        text = pytesseract.image_to_string(
            Image.open(image_path), lang=self.ocr_language
        )
        return text

    def ocr_screen_region(self, region):
        """Perform OCR on a specific region of the screen.
        Region should be a tuple: (x, y, width, height).
        """
        screenshot = pyautogui.screenshot(region=region)
        text = pytesseract.image_to_string(screenshot, lang=self.ocr_language)
        return text

    def capture_screen_region(self, region, path):
        """Capture a specific region of the screen and save it to the specified path.
        Region should be a tuple: (x, y, width, height).
        """
        self.take_screenshot(path, region=region)

    def get_pixel_color(self, x, y):
        """Get the color of a specific pixel on the screen."""
        pixel_color = pyautogui.pixel(x, y)
        return pixel_color

    def find_and_click_text(self, text):
        """Find and click on text on the screen using OCR."""
        screenshot = pyautogui.screenshot()
        img = Image.fromarray(screenshot)
        boxes = pytesseract.image_to_boxes(img, lang=self.ocr_language)
        for box in boxes.splitlines():
            b = box.split(' ')
            if b[0] == text:
                x, y, w, h = int(b[1]), int(b[2]), int(b[3]), int(b[4])
                pyautogui.click(x + w // 2, y + h // 2)
                return
        raise ValueError(f"Text '{text}' not found on screen")

    def record_screen(self, path, duration):
        """Record the screen for a specified duration and save it to the specified path."""
        try:
            import cv2
            import numpy as np

            screen_size = pyautogui.size()
            fourcc = cv2.VideoWriter_fourcc(*'XVID')
            out = cv2.VideoWriter(
                str(path),
                fourcc,
                20.0,
                (screen_size.width, screen_size.height),
            )

            start_time = time.time()
            while time.time() - start_time < duration:
                img = pyautogui.screenshot()
                frame = np.array(img)
                frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                out.write(frame)

            out.release()
        except Exception:
            print('Screen recording is not supported on this OS.')

    def monitor_screen_region(
        self, region, action_on_change, timeout=None, max_iterations=None
    ):
        """Monitor a region of the screen and execute an action when changes are detected.
        Region should be a tuple: (x, y, width, height).
        The function will stop monitoring after the specified timeout (in seconds) or
        after the specified number of iterations.
        """
        start_time = time.time()
        iterations = 0
        initial_screenshot = pyautogui.screenshot(region=region)

        while (timeout is None or (time.time() - start_time) <= timeout) and (
            max_iterations is None or iterations < max_iterations
        ):
            current_screenshot = pyautogui.screenshot(region=region)
            if initial_screenshot != current_screenshot:
                action_on_change()
                initial_screenshot = current_screenshot

            iterations += 1
            time.sleep(1)
